;********************************************************************
;
; (C) Copyright notice
;
; Written by Richard Prinz.
; Copyright 2005 by Richard Prinz.
; OE1RIB Richard.Prinz@MIN.at
;
; Permission to use, copy, and modify this program without restriction 
; is hereby granted, as long as
;
; 1) this copyright notice appears in each copy of the program source 
;    code and
;
; 2) the source code is not used for commercial purposes.
;
; This program is freely distributable without licensing fees and is 
; provided without guarantee or warrantee expressed or implied.
;
; This program is -NOT- in the public domain.
;
;********************************************************************


	; ==============================================================
			list				p=16F819
#include		<P16F819.inc>
#include		"I2C-Servo.h"
	; ==============================================================
			__CONFIG			_CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC & _MCLR_OFF & _LVP_ON
			__IDLOCS			0xCAFE
	; ==============================================================		
_DEVID1		EQU				H'3FFFFE'
_DEVID2		EQU				H'3FFFFF'
	; ==============================================================		
			udata_shr
ISR_STATUS	res				1
ISR_W		res				1
FSR_TEMP		res 				1
			global			FSR_TEMP
ServoScratch	res				1
	; -------------------------------------
			udata
WORK			res				1
TEMP			res				1
TMR0H		res				1
MASK			res				1
PULSE		res				1
BasePULSE		res				1
TRIM			res				1
ServoStatus	res				1
			global			ServoStatus
TempServoIndex	res				1
			global			TempServoIndex
TempServoCmd	res				1
			global			TempServoCmd
TempServoValue	res				1
			global			TempServoValue
ServoIndex	res				1
ServosActive	res				1
			global			ServosActive
Servo0Preset	res				16
			global			Servo0Preset
	; ==============================================================		
			extern			I2C_Init, I2C_Handler			
			extern			RestoreTrim, RestorePositions, SaveTrim, SavePositions
			extern			ReadConfigByte, WriteConfigByte
			extern			ReadOnOffMask, WriteOnOffMask, RestoreOnOffMask
	; ==============================================================		

;************************************************** 
;
;       Definitions
;
;**************************************************

; FOSC = 40000000
; T0_PRESCALER = 256
; NUM_OF_SERVOS = 8
;
; TIME_PER_SERVO = ((1 / 50) / NUM_OF_SERVOS)		; 0,0025
; TIMER = (1 / (FOSC / 4)) * T0_PRESCALER			; 0,0000512

; TIMER_PRESET = 65535 - (TIME_PER_SERVO / TIMER)		; 16707		48,828125		49	207
; PRESET_HI = High TIMER_PRESET					; 0x41
; PRESET_LO = Low TIMER_PRESET					; 0x43

PresetTimer macro
			banksel			TMR0
			movlw			0xCF				; PRESET_HI
			movwf			TMR0
	endm

;************************************************** 
;
;       Reset/IRQ Vectors
;
;**************************************************
_Start		code				0x0000			
			goto				Configuration
_ISR			code				0x0004
			goto				ISR
_Main		code

;**************************************************
;
;       Interrup service routines
;
;**************************************************
ISR
			; save original W and STATUS registers
			movwf			ISR_W			; 1
			swapf			STATUS, w			; 1
			clrf				STATUS			; 1
			movwf			ISR_STATUS		; 1

			banksel			INTCON			; 2						
			btfsc			INTCON, TMR0IF		; 1 / 2
			goto				T0_IRQ			; 2
			
			banksel			PIR1				; 2
			btfsc			PIR1, SSPIF		; 1 / 2
			goto				I2C_IRQ			; 2
			
			goto				ISR_End			; 2
;------------------------------------------------------			
T0_IRQ										;--> 11
			banksel			ServoStatus		; 1
			bsf				ServoStatus, TIMER_EVENT	; 1
			PresetTimer						; 2
			banksel			INTCON			; 2
			bcf				INTCON, TMR0IF		; 1
			goto				ISR_End			; 2
;------------------------------------------------------			
I2C_IRQ
			;banksel			SSPSTAT
			;btfss			SSPSTAT, BF
			;goto			I2C_IRQ_1			
			;banksel			SSPCON
			;bcf				SSPCON, CKP
I2C_IRQ_1						
			call				I2C_Handler
			banksel			PIR1				; 2
			bcf				PIR1, SSPIF		; 1
	
;------------------------------------------------------			
ISR_End
			; restore W and STATUS registers
			swapf			ISR_STATUS, w		; 1
			movwf			STATUS			; 1
			swapf			ISR_W, f			; 1
			swapf			ISR_W, w			; 1
			retfie							; 2
											;---
											; 8

;**************************************************
;
;       Main program
;
;**************************************************

;--------------------------------------------------
;		basic IO and HW configuration
;--------------------------------------------------
Configuration
#include		"Config819.asm"



;--------------------------------------------------
;		start main task
;--------------------------------------------------
Start
			call				I2C_Init

			PresetTimer

			banksel			INTCON
			bcf				INTCON, GIE
			bcf				INTCON, TMR0IF
			bsf				INTCON, TMR0IE
			bsf				INTCON, PEIE
			
			banksel			TEMP
			clrf				TEMP

			movlw			B'10000000'
			movwf			MASK
			
			clrf				ServoIndex

InitSystem
			call				SetPositionDefaults
			call				SetTrimDefaults
			banksel			ServosActive
			movlw			0xFF
			movwf			ServosActive

			call				ReadConfigByte
			banksel			TEMP
			movwf			TEMP
			
			btfss			TEMP, isConfigured
			goto				InitEEPROM

			btfsc			TEMP, StartWithTrim
			call				RestoreTrim
			
			banksel			TEMP
			btfsc			TEMP, StartWithPosition
			call				RestorePositions
			
			banksel			TEMP
			btfsc			TEMP, StartWithOnOffMask
			call				RestoreOnOffMask
	
			goto				StartSystem
			
InitEEPROM
			call				SaveTrim
			call				SavePositions
			banksel			TEMP
			clrf				TEMP
			bsf				TEMP, isConfigured
			movf				TEMP, w
			call				WriteConfigByte
			movlw			0xFF
			call				WriteOnOffMask

StartSystem
			banksel			INTCON
			bsf				INTCON, GIE



;--------------------------------------------------
;		Main Loop
;--------------------------------------------------
Main
			banksel			ServoStatus
			btfss			ServoStatus, TIMER_EVENT
			goto				Main
			bcf				ServoStatus, TIMER_EVENT

;--------------------------------------------------
;		Timer Handler / Servo Loop
;--------------------------------------------------
ServoTaskInit

			; debug code
			; toggles PORTB,0 every cycle = 2,5ms
			;btfss			WORK, 0			; 1 / 2
			;bcf				STATUS, C			; 1
			;btfsc			WORK, 0			; 1 / 2
			;bsf				STATUS, C			; 1
											; ----- 4

			;banksel			PORTB			; 2
			
			;btfss			STATUS, C			; 1 / 2
			;bcf				DEBUG_PIN			; 1
			;btfsc			STATUS, C			; 1 / 2
			;bsf				DEBUG_PIN			; 1
											; ----- 4
										
			;banksel			ServoIndex		; 2
											; ----- 12
			; end of debug code

			
			movf				ServoIndex, w		; 1
			btfss			STATUS, Z			; 1 / 2
			goto				ServoTask			; 2
			SetFSR			Servo0Preset		; 3
											; ----- 4 / 5
ServoTask	
			movf				INDF, w			; 1	ServoTrim
			movwf			TRIM				; 1
			incf				FSR, f			; 1
			
			movf				INDF, w			; 1	ServoPosition
			movwf			PULSE			; 1
			
			; zero clamp
			btfsc			STATUS, Z			; +
			incf				PULSE, f			; = 2
			; ----------
			
			incf				FSR, f			; 1
			
			movlw			0xFF				; 1
			movwf			BasePULSE			; 1
											; ----- 8

			btfss			TRIM, 7			; 1 / 2
			goto				ServoTask_1		; 2
			
			movlw			B'01111111'		; 1
			andwf			TRIM, w			; 1
			subwf			BasePULSE, f		; 1
											; ----- 5 / 6
ServoTask_1
			bcf				STATUS, C			; 1
			btfsc			MASK, 7			; +
			bsf				STATUS, C			; = 2
			rlf				MASK, f			; 1
			movf				MASK, w			; 1
			andwf			ServosActive, w	; 1
											; ----- 6
	;--------------------------------------------------
	
			movwf			ServoScratch		; 1
			banksel			PORTA			; 2
			movwf			PORTA			; 1
			btfsc			ServoScratch, 6	; +
			bsf				Servo6			; = 2
			btfsc			ServoScratch, 7	; +
			bsf				Servo7			; = 2
			banksel			ServoIndex		; 2
											; ----- 10
ServoPrePulse
			call				WaitStep			; 16
			nop								; 1
			decfsz			BasePULSE, f		; +
			goto				ServoPrePulse		; +
			nop								; = 3
											; ----- 20 * 256 = 5120 = 1,02ms
ServoPulse
			call				WaitStep			; 16
			nop								; 1
			decfsz			PULSE, f			; +
			goto				ServoPulse		; +
			nop								; = 3
											; ----- 20 * PULSE

			btfsc			TRIM, 7			; 1 / 2
			goto				ServoDone			; 2
			movf				TRIM, w			; 1
			movwf			BasePULSE			; 1
											; ----- 4 / 5
ServoPostPulse
			call				WaitStep			; 16
			nop								; 1
			decfsz			BasePULSE, f		; +
			goto				ServoPostPulse		; +
			nop								; = 3
											; ----- 20 * PULSE			
ServoDone			
			banksel			PORTA			; 2
			clrf				PORTA			; 1
			bcf				Servo6			; 1
			bcf				Servo7			; 1
											; ----- 5

	;--------------------------------------------------
	
ServoTaskInc
			banksel			ServoIndex		; 2
			incf				ServoIndex, w		; 1
			andlw			B'00000111'		; 1
			movwf			ServoIndex		; 1
			goto				Main				; 2
											; ----- 7


	; --------------------------------------------------------------
	; ***	WaitStep
	; ***	waits for 16 instructions = 3,2uS (inkl call and return)
WaitStep										; 2
			movlw			.3				; 1
			movwf			ServoScratch		; 1
			nop								; 1
WaitStep_1	
			decfsz			ServoScratch, f	; +
			goto				WaitStep_1		; +
			nop								; = 3
			
			return							; 2
			
	; --------------------------------------------------------------
	; ***	SetPositionsDefaults
	; ***	Sets all servo positions to center
SetPositionDefaults
			global			SetPositionDefaults
			SetFSR			Servo0Preset
			incf				FSR, f

			banksel			TEMP
			movlw			.8
			movwf			TEMP			
SetPositionDefaults_Loop			
			movlw			0x80
			movwf			INDF
			incf				FSR, f
			incf				FSR, f
			
			decfsz			TEMP, f
			goto				SetPositionDefaults_Loop
SetPositionDefaults_Done
			return

	; --------------------------------------------------------------
	; ***	SetTrimDefaults
	; ***	Sets all servo trims to center
SetTrimDefaults
			global			SetTrimDefaults
			SetFSR			Servo0Preset

			banksel			TEMP
			movlw			.8
			movwf			TEMP
SetTrimDefaults_Loop
			movlw			0x80
			movwf			INDF
			incf				FSR, f			
			incf				FSR, f
			
			decfsz			TEMP, f
			goto				SetTrimDefaults_Loop
SetTrimDefaults_Done
			return			



.eedata		code				0x2100 
EE_Config		DE				0x00

			end


